import { Controller } from '@hotwired/stimulus';
import attachSlimSelectTo from '../attach_slimselect';
import { isEmailAddress } from '../emails';

export default class extends Controller {
  static targets = ['recipientSelect', 'submitButton'];

  connect() {
    this.requiredFields = this.element.querySelectorAll('*[required]');

    this.detachSlimSelects = [];
    this.recipientSelectTargets.forEach(el => {
      const detachCallback = attachSlimSelectTo(el, {
        events: {
          addable: value => {
            return isEmailAddress(value) && value;
          },
        },
      });

      this.detachSlimSelects.push(detachCallback);
    });

    this.requiredFields.forEach(el => {
      el.addEventListener('valueChanged', this.updateButtonVisibility.bind(this));
      el.addEventListener('keyup', this.updateButtonVisibility.bind(this));
    });
  }

  disconnect() {
    this.requiredFields.forEach(el => {
      el.removeEventListener('valueChanged', this.updateButtonVisibility.bind(this));
      el.removeEventListener('keyup', this.updateButtonVisibility.bind(this));
    });

    this.detachSlimSelects.forEach(callback => callback(this));
  }

  addRadioListeners() {}

  removeRadioListeners() {}

  updateButtonVisibility(ev) {
    var shouldShowButton = true;

    this.requiredFields.forEach(el => {
      shouldShowButton &&= el.value.length > 0;
    });

    this.submitButtonTarget.classList.toggle('hidden', !shouldShowButton);
  }
}
