import { Given, When, Then } from '@badeball/cypress-cucumber-preprocessor';

let tagset_ids = [];
let directory_id = null;

Given('There is a directory with some docs', () => {
  cy.appFactories([['create_list', 'tagset_doc', 5, 'with_state']]).then(records => {
    records[0].forEach(record => {
      tagset_ids.push(record.id);
    });
  });
  cy.appFactories([['create', 'tagset_directory', 'with_logs', { responsible: '1', id: '1', tagset_ids: tagset_ids }]]).then(records => {
    directory_id = records[0].id;
  });
});

When('I visit the directory url', () => {
  cy.visit(`/spirit_hub/directories/${directory_id}/index`);
});

Given('There are existing users I want to share a doc with', () => {
  cy.appFactories([['create', 'user', { email: 'john@test.com', first_name: 'John', last_name: 'Doe' }]]);
  cy.appFactories([['create', 'user', { email: 'jack@test.com', first_name: 'Jack', last_name: 'Black' }]]);
  cy.appFactories([['create', 'user', { email: 'josh@test.com', first_name: 'Josh', last_name: 'Homme' }]]);
});

Given('There are existing user groups I want to share a doc with', () => {
  const uids = {};

  cy.appFactories([['create', 'user', { email: 'john@test.com', first_name: 'John', last_name: 'Doe' }]]).then(records => {
    uids.user1 = records[0].id;
  });
  cy.appFactories([['create', 'user', { email: 'jack@test.com', first_name: 'Jack', last_name: 'Black' }]]).then(records => {
    uids.user2 = records[0].id;
  });
  cy.appFactories([['create', 'user', { email: 'josh@test.com', first_name: 'Josh', last_name: 'Homme' }]]).then(records => {
    uids.user3 = records[0].id;

    cy.appFactories([['create', 'group', { name: 'Group One', users_ids: [uids.user1, uids.user2] }]]);
    cy.appFactories([['create', 'group', { name: 'Group Two', users_ids: [uids.user3] }]]);
  });
});

Given('There are existing external contacts I want to share a doc with', () => {
  cy.appFactories([['create', 'contact', { title: 'john@test.com', superfields: { given_name: 'John', surname: 'Doe' } }]]);
  cy.appFactories([['create', 'contact', { title: 'jack@test.com', superfields: { given_name: 'Jack', surname: 'Black' } }]]);
  cy.appFactories([['create', 'contact', { title: 'josh@test.com', superfields: { given_name: 'Josh', surname: 'Homme' } }]]);
});

Then('I can share the doc with the existing users', option => {
  cy.get('form[action*="/spirit_hub/tagsets/form/share"] button[type="submit"]').first().click();

  cy.get('select[name="to[]"]').next('div').click();
  cy.contains('.ss-list:visible > .ss-option', 'John Doe (john@test.com)').first().click({ force: true });
  cy.get('select[name="cc[]"]').next('div').click();
  cy.contains('.ss-list:visible > .ss-option', 'Jack Black (jack@test.com)').first().click({ force: true });
  cy.get('select[name="bcc[]"]').next('div').click();
  cy.contains('.ss-list:visible > .ss-option', 'Josh Homme (josh@test.com)').first().click({ force: true });
  cy.get('input[name="subject"]').first().type('Hey');
  cy.get('textarea[name="message"]').first().type('Hello there!');
  cy.get('form[action*="/share"] button[type="submit"]').first().click();

  cy.get('div[id*="informed_badges"]').first().find('img').should('have.length', 3);
  cy.get('div[id*="informed_badges"] img[src*="JD"]').should('exist');
  cy.get('div[id*="informed_badges"] img[src*="JB"]').should('exist');
  cy.get('div[id*="informed_badges"] img[src*="JH"]').should('exist');
});

Then('I can share the doc with the existing user groups', option => {
  cy.get('form[action*="/spirit_hub/tagsets/form/share"] button[type="submit"]').first().click();

  cy.get('select[name="to[]"]').next('div').click();
  cy.contains('.ss-list:visible > .ss-option', 'Groupe: Group One (2 utilisateurs)').first().click({ force: true });
  cy.get('select[name="cc[]"]').next('div').click();
  cy.contains('.ss-list:visible > .ss-option', 'Groupe: Group Two (1 utilisateurs)').first().click({ force: true });
  cy.get('input[name="subject"]').first().type('Hey');
  cy.get('textarea[name="message"]').first().type('Hello there!');
  cy.get('form[action*="/share"] button[type="submit"]').first().click();

  cy.get('div[id*="informed_badges"]').first().find('img').should('have.length', 3);
  cy.get('div[id*="informed_badges"] img[src*="JD"]').should('exist');
  cy.get('div[id*="informed_badges"] img[src*="JB"]').should('exist');
  cy.get('div[id*="informed_badges"] img[src*="JH"]').should('exist');
});

Then('I can share the doc with the external contacts', option => {
  cy.get('form[action*="/spirit_hub/tagsets/form/share"] button[type="submit"]').first().click();

  cy.get('select[name="to[]"]').next('div').click();
  cy.contains('.ss-list:visible > .ss-option', 'John Doe (john@test.com)').first().click({ force: true });
  cy.get('select[name="cc[]"]').next('div').click();
  cy.contains('.ss-list:visible > .ss-option', 'Jack Black (jack@test.com)').first().click({ force: true });
  cy.get('select[name="bcc[]"]').next('div').click();
  cy.contains('.ss-list:visible > .ss-option', 'Josh Homme (josh@test.com)').first().click({ force: true });
  cy.get('input[name="subject"]').first().type('Hey');
  cy.get('textarea[name="message"]').first().type('Hello there!');
  cy.get('form[action*="/share"] button[type="submit"]').first().click();

  cy.get('div[id*="informed_badges"]').first().find('.rounded-full').should('have.length', 3);
  cy.get('div[id*="informed_badges"] .rounded-full').contains('JD').should('exist');
  cy.get('div[id*="informed_badges"] .rounded-full').contains('JB').should('exist');
  cy.get('div[id*="informed_badges"] .rounded-full').contains('JH').should('exist');
});

Then('I can share the doc with new external contacts', option => {
  cy.get('form[action*="/spirit_hub/tagsets/form/share"] button[type="submit"]').first().click();

  cy.get('select[name="to[]"]').next('div').click();
  cy.get('.ss-search:visible input[type="search"]').first().type('invalid@email');
  cy.get('.ss-search:visible .ss-addable').first().click();
  cy.get('.ss-value-text:visible').should('not.exist'); // Does not accept invalid emails

  cy.get('.ss-search:visible input[type="search"]').first().clear();
  cy.get('.ss-search:visible input[type="search"]').first().type('john.doe@gmail.com');
  cy.get('.ss-search:visible .ss-addable').first().click();
  cy.get('.ss-value-text:visible').contains('john.doe@gmail.com').should('exist');

  cy.get('input[name="subject"]').first().type('Hey');
  cy.get('textarea[name="message"]').first().type('Hello there!');
  cy.get('form[action*="/share"] button[type="submit"]').first().click();

  cy.get('div[id*="informed_badges"]').first().find('.rounded-full').should('have.length', 1);
  cy.get('div[id*="informed_badges"] .rounded-full').contains('J').should('exist');
});

Then('I can share the doc with the user groups', option => {
  cy.get('form[action*="/spirit_hub/tagsets/form/share"] button[type="submit"]').first().click();

  cy.get('select[name="to[]"]').next('div').click();
  cy.contains('.ss-list > .ss-option', 'John Doe (john@test.com)').first().click({ force: true });
  cy.get('select[name="cc[]"]').next('div').click();
  cy.contains('.ss-list > .ss-option', 'Jack Black (jack@test.com)').first().click({ force: true });
  cy.get('select[name="to[]"]').next('div').click();
  cy.contains('.ss-list > .ss-option', 'Josh Homme (josh@test.com)').first().click({ force: true });
  cy.get('input[name="subject"]').first().type('Hey');
  cy.get('textarea[name="message"]').first().type('Hello there!');
  cy.get('form[action*="/share"] button[type="submit"]').first().click();

  cy.get('div[id*="informed_badges"]').first().find('img').should('have.length', 3);
  cy.get('div[id*="informed_badges"] img[src*="JD"]').should('exist');
  cy.get('div[id*="informed_badges"] img[src*="JB"]').should('exist');
  cy.get('div[id*="informed_badges"] img[src*="JH"]').should('exist');
});
